@@ -8,15 +8,15 @@ from betterproto import Casing
 
 from maitai._config import config
 from maitai._maitai_client import MaitaiClient
-from maitai._utils import chat_completion_chunk_to_response
+from maitai._utils import __version__ as version, chat_completion_chunk_to_response
 from maitai_common.utils.types import EvaluateCallback
-from maitai_common.version import version
 from maitai_gen.chat import ChatCompletionChunk, ChatCompletionParams, ChatCompletionResponse, EvaluateRequest, EvaluateResponse, EvaluationContentType
+from maitai_gen.metric import RequestTimingMetric
 
 
 def _get_content_type(content, partial=None):
     if partial is not None:
-        return EvaluateResponse.PARTIAL_CONTENT
+        return EvaluationContentType.PARTIAL
     if isinstance(content, str):
         return EvaluationContentType.TEXT
     elif isinstance(content, list):
@@ -30,7 +30,7 @@ class Evaluator(MaitaiClient):
 
     @classmethod
     async def evaluate_async(cls, session_id, reference_id, intent, content, content_type=None, application_id=None, application_ref_name=None, callback=None, completion_params: ChatCompletionParams = None,
-                             chat_completion_response: ChatCompletionResponse = None, chat_completion_chunk: ChatCompletionChunk = None):
+                             chat_completion_response: ChatCompletionResponse = None, chat_completion_chunk: ChatCompletionChunk = None, timing: RequestTimingMetric = None):
         if content_type is None:
             content_type = _get_content_type(content)
         if content_type is None:
@@ -43,12 +43,12 @@ class Evaluator(MaitaiClient):
             if chat_completion_chunk is not None:
                 chat_completion_response = chat_completion_chunk_to_response(chat_completion_chunk, content)
             eval_request.chat_completion_response = chat_completion_response
-
+        eval_request.timing_metrics = timing
         await asyncio.create_task(cls.send_evaluation_request_async(eval_request, callback))
 
     @classmethod
     def evaluate(cls, session_id, reference_id, intent, content, content_type=None, application_id=None, application_ref_name=None, callback=None, partial=None, completion_params: ChatCompletionParams = None,
-                 chat_completion_response: ChatCompletionResponse = None, chat_completion_chunk: ChatCompletionChunk = None):
+                 chat_completion_response: ChatCompletionResponse = None, chat_completion_chunk: ChatCompletionChunk = None, timing: RequestTimingMetric = None):
         if content_type is None:
             content_type = _get_content_type(content, partial)
         if content_type is None:
@@ -57,10 +57,11 @@ class Evaluator(MaitaiClient):
             raise Exception('application_id or application_ref_name must be provided')
         eval_request: EvaluateRequest = cls.create_eval_request(application_id, application_ref_name, session_id, reference_id, intent, content_type, content, partial=partial)
         if completion_params is not None:
-            eval_request.chat_completion_request = cls.create_inference_request(application_ref_name, session_id, reference_id, intent, False, True, completion_params, callback, False, '')
+            eval_request.chat_completion_request = cls.create_inference_request(application_ref_name, session_id, reference_id, intent, False, True, completion_params, callback, False, None)
             if chat_completion_chunk is not None:
                 chat_completion_response = chat_completion_chunk_to_response(chat_completion_chunk, content)
             eval_request.chat_completion_response = chat_completion_response
+        eval_request.timing_metrics = timing
 
         cls.run_async(cls.send_evaluation_request_async(eval_request, callback))
 
