@@ -1,16 +1,20 @@
 import functools
 import inspect
-from typing import Any, Callable, Dict, Iterable, List, Optional, Sequence, Tuple, TypeVar, Union
+import sys
+from typing import Any, Callable, Dict, Iterable, List, Optional, Sequence, TypeVar, Union
 
 import openai
 import openai.types.chat as openai_types
+from betterproto import Casing
 from openai.types import ChatModel
 
 import maitai_gen.chat as chat_types
-from maitai._evaluator import Evaluator
-from maitai._types import EvaluateCallback
+from maitai._types import ToolFunction
+from maitai.tools import Tools
 from maitai_common.utils.proto_utils import openai_messages_to_proto
-from maitai_gen.chat import ChatCompletionParams, Function, ResponseFormat, StreamOptions, Tool
+from maitai_common.version import version
+
+__version__ = f'{version}_python_{sys.version_info.major}.{sys.version_info.minor}.{sys.version_info.micro}'
 
 CallableT = TypeVar("CallableT", bound=Callable[..., Any])
 
@@ -93,13 +97,6 @@ def convert_openai_chat_completion(chat: openai_types.ChatCompletion) -> chat_ty
     return chat_types.ChatCompletionResponse().from_dict(chat.to_dict())
 
 
-def _copy_choice(choice: Union[openai_types.chat_completion.Choice, chat_types.ChatCompletionChoice]) -> chat_types.ChatCompletionChoice:
-    message = chat_types.ChatMessage(content=choice.message.content, role=choice.message.role)
-    copied_choice = chat_types.ChatCompletionChoice(finish_reason=choice.finish_reason, index=choice.index,
-                                                    logprobs=choice.logprobs, message=message)
-    return copied_choice
-
-
 def convert_open_ai_chat_completion_chunk(chunk: openai_types.ChatCompletionChunk) -> chat_types.ChatCompletionChunk:
     return chat_types.ChatCompletionChunk().from_dict(chunk.to_dict())
 
@@ -113,19 +110,19 @@ def get_chat_completion_params(*,
                                max_tokens: Union[Optional[int], openai.NotGiven] = openai.NOT_GIVEN,
                                n: Union[Optional[int], openai.NotGiven] = openai.NOT_GIVEN,
                                presence_penalty: Union[Optional[float], openai.NotGiven] = openai.NOT_GIVEN,
-                               response_format: Union[openai_types.completion_create_params.ResponseFormat, openai.NotGiven] = openai.NOT_GIVEN,
+                               response_format: Union[openai_types.completion_create_params.ResponseFormat, chat_types.ResponseFormat, openai.NotGiven] = openai.NOT_GIVEN,
                                seed: Union[Optional[int], openai.NotGiven] = openai.NOT_GIVEN,
                                stop: Union[Union[Optional[str], List[str]], openai.NotGiven] = openai.NOT_GIVEN,
                                stream: Optional[bool] = False,
                                stream_options: Union[Optional[openai_types.ChatCompletionStreamOptionsParam], openai.NotGiven] = openai.NOT_GIVEN,
                                temperature: Union[Optional[float], openai.NotGiven] = openai.NOT_GIVEN,
                                tool_choice: Union[openai_types.ChatCompletionToolChoiceOptionParam, openai.NotGiven] = openai.NOT_GIVEN,
-                               tools: Union[Iterable[openai_types.ChatCompletionToolParam], openai.NotGiven] = openai.NOT_GIVEN,
+                               tools: Union[Iterable[Union[openai_types.ChatCompletionToolParam, ToolFunction]], openai.NotGiven] = openai.NOT_GIVEN,
                                top_logprobs: Union[Optional[int], openai.NotGiven] = openai.NOT_GIVEN,
                                top_p: Union[Optional[float], openai.NotGiven] = openai.NOT_GIVEN,
                                user: Union[str, openai.NotGiven] = openai.NOT_GIVEN,
-                               ) -> ChatCompletionParams:
-    params = ChatCompletionParams(
+                               ) -> chat_types.ChatCompletionParams:
+    params = chat_types.ChatCompletionParams(
         messages=openai_messages_to_proto(messages),
         model=model,
         stream=stream,
@@ -152,51 +149,42 @@ def get_chat_completion_params(*,
             setattr(params, param_name, value)
 
     if stream_options != openai.NOT_GIVEN and stream_options:
-        params.stream_options = StreamOptions(include_usage=stream_options.get("include_usage"))
+        params.stream_options = chat_types.StreamOptions(include_usage=stream_options.get("include_usage"))
 
     if response_format != openai.NOT_GIVEN:
-        params.response_format = ResponseFormat(type=response_format["type"])
+        params.response_format = chat_types.ResponseFormat(type=response_format["type"])
+        if response_format.get("json_schema"):
+            params.response_format.json_schema = chat_types.JsonSchema().from_dict(response_format["json_schema"])
 
     if tools != openai.NOT_GIVEN:
         params.tools = []
-        for tool in tools:
-            function = tool.get("function")
-            params.tools.append(
-                Tool(
-                    type=tool["type"],
-                    function=Function(
-                        name=function.get("name"),
-                        description=function.get("description"),
-                        parameters=function.get("parameters")
+        if isinstance(tools, Tools):
+            params.tools = tools.get_tool_definitions()
+        else:
+            for tool in tools:
+                if isinstance(tool, Callable) and not isinstance(tool, ToolFunction):
+                    raise TypeError(f"Method passed into tools missing @tool decorator")
+                if isinstance(tool, ToolFunction):
+                    params.tools.append(tool.__tool__)
+                    continue
+                function = tool.get("function")
+                params.tools.append(
+                    chat_types.Tool(
+                        type=tool["type"],
+                        function=chat_types.Function(
+                            name=function.get("name"),
+                            description=function.get("description"),
+                            parameters=function.get("parameters")
+                        )
                     )
                 )
-            )
     return params
 
 
-def process_chunk(session_id: Union[str, int],
-                  reference_id: Union[str, int, None],
-                  action_type: str,
-                  application_ref_name: str,
-                  proto_messages: list,
-                  full_body: str,
-                  maitai_chunk: chat_types.ChatCompletionChunk,
-                  callback: Optional[EvaluateCallback]) -> Tuple[str, Optional[chat_types.ChatCompletionChunk]]:
-    content = maitai_chunk.choices[0].delta.content
-    if content is not None:
-        full_body += content
-    if maitai_chunk.choices[0].finish_reason:
-        proto_messages.append(chat_types.ChatMessage(role="assistant", content=full_body))
-        maitai_eval = Evaluator.evaluate(
-            session_id=session_id,
-            reference_id=reference_id,
-            action_type=action_type,
-            content_type=chat_types.EvaluationContentType.MESSAGE,
-            content=proto_messages,
-            application_ref_name=application_ref_name,
-            callback=callback
-        )
-        if not callback:
-            maitai_chunk.evaluate_response = maitai_eval
-            return full_body, maitai_chunk
-    return full_body, None
+def chat_completion_chunk_to_response(final_chunk: chat_types.ChatCompletionChunk, content: Union[str, List[chat_types.ChatMessage]]):
+    chat_completion_response = chat_types.ChatCompletionResponse().from_dict(final_chunk.to_dict(casing=Casing.SNAKE))
+    if isinstance(content, str):
+        chat_completion_response.choices[0].message = chat_types.ChatMessage(role='assistant', content=content)
+    if isinstance(content, List):
+        chat_completion_response.choices[0].message = content[-1]
+    return chat_completion_response
